#
#	publish_on_pages.sh
#
#	Script to publish specs on gitlab pages supporting multiversioning (content.zip approach) 
#
#	(c) 2024 by Miguel Angel Reina Ortega
#	License: BSD 3-Clause License. See the LICENSE file for further details.
#
#!/bin/bash

DOCKER_IMAGE=logiqx/python-bs4:3.9-slim

echo "\n------ Checking for docker image --------"
docker pull "$DOCKER_IMAGE"
echo "Pages URL:" $2
echo "PROJECT NAME:" $3 # Not used anymore, consider to remove it
echo "TAG NAME:" $4

echo "------ Get the previous GitLab Pages content --------"
set +e
curl --fail "$2/content.zip" -o "content.zip"
echo $BACKUP
if [ $BACKUP == "true" ]; then
    echo "Retrieving content.zip from backup"
    curl "https://git.onem2m.org/specifications/ts-0019/-/jobs/1082/artifacts/download?file_type=archive" >> content.zip
    unzip "content.zip";
    rm "content.zip"
elif [ -e "content.zip" ]; then
    unzip "content.zip";
    rm "content.zip";
else
    if [ $4 == 'upgrade' ]; then
        echo 'Pages do not exist yet';
        exit 0
    fi
fi
set -e

echo "------ Get the previous GitLab Pages content --------"
# Move all folders except the "baselines" folder if baselines folder exists
if [ -d "public/baselines" ]; then
    echo "Baselines already moved to baselines/";
else
    mkdir -p "public/baselines"
    for folder in public/*; do
        if [ -d "$folder" ] && [ "$(basename "$folder")" != "baselines" ]; then
            mv "$folder" "public/baselines/"
            echo "Moved $(basename "$folder") to public/baselines"
        fi
    done
    # Move index.html with new location
    mv "public/index.html" "public/baselines/"

fi

echo "------ Add/update content --------"
if [ $4 == 'upgrade' ]; then
    echo 'Upgrading index.html...'
    docker run --rm -v $(pwd):/tmp -w /tmp "$DOCKER_IMAGE" python3 $1 -u "$7_$4" "$4/$7_$4.docx" "$5"
elif [ $6 == 'true' ]; then
    echo 'Removing all baselines ...'
	rm -r public/baselines/*;
	mkdir -p "public/baselines/$4"
    ls public/
	cp -r -f "$7_$4.docx" public/baselines/$4
    docker run --rm -v $(pwd):/tmp -w /tmp "$DOCKER_IMAGE" python3 $1 "$7_$4" "$4/$7_$4.docx" "$5"
elif [ $6 == 'false' ]; then
	mkdir -p "public/baselines/$4"
    ls public/
	cp -r -f "$7_$4.docx" public/baselines/$4
    docker run --rm -v $(pwd):/tmp -w /tmp "$DOCKER_IMAGE" python3 $1 "$7_$4" "$4/$7_$4.docx" "$5"
else
    echo 'Removing entry for ' $6
    rm -r -f public/baselines/$6
    docker run --rm -v $(pwd):/tmp -w /tmp "$DOCKER_IMAGE" python3 $1 -re "$7_$4" "$4/$7_$4.docx" "$5"
fi

echo "------ Zip the content and publish the zip again ------"
zip -r "content.zip" "public"
mv "content.zip" "public/"
 
exit 0

#Parameters
#updateIndex.py 
#$CI_PAGES_URL
#$CI_PROJECT_NAME
#$CI_COMMIT_REF_NAME
#index.html
#CLEAN_WEB_PAGES
#SPEC_NAME
