#
#	spec_on_pages.sh
#
#	Script to publish specs on gitlab pages supporting multiversioning (gl-pages+Mike approach) 
# It handles multiversioning by setting latest version and hidding older versions (MinorVersion) per release (Release)
#   v<Release>.<IntermediateVersion>.<MinorVersion>
#
#	(c) 2024 by Miguel Angel Reina Ortega
#	License: BSD 3-Clause License. See the LICENSE file for further details.
#
#!/bin/bash

#Parameters
#toMkdocs.py -> 1
echo "toMkdocs.py:" $1
#$MIKE_ACCESS_TOKEN -> 2
echo "MIKE_ACCESS_TOKEN:" $2
#CI_SERVER_HOST -> 3
echo "CI_SERVER_HOST:" $3
#CI_PROJECT_PATH -> 4
echo "CI_PROJECT_PATH:" $4
#GITLAB_USER_NAME -> 5
echo "GITLAB_USER_NAME:" $5
#GITLAB_USER_EMAIL -> 6
echo "GITLAB_USER_EMAIL:" $6
#CI_COMMIT_SHA -> 7
echo "CI_COMMIT_SHA:" $7
#$SPEC_NAME -> 8
echo "SPEC_NAME:" $8
#$CI_PROJECT_NAME -> 9
echo "CI_PROJECT_NAME:" $9
#$TAG_NAME ->10
echo "TAG_NAME:" ${10}
#$CLEAN_WEB_PAGES ->11
echo "CLEAN_WEB_PAGES:" ${11}

#BACKUP=false
PAGES_BRANCH=gl-pages
HTTPS_REMOTE=https://gitlab-ci-token:${2}@${3}/${4}.git

echo "\n------ Install necessary packages --------"
pip install -q mkdocs-material mike rich mkdocs-rss-plugin

echo "------ Config git and get the previous GitLab Pages content --------"
git config --global --replace-all user.name "$5"
git config --global --replace-all user.email $6
git fetch origin $PAGES_BRANCH && git checkout $PAGES_BRANCH || git checkout -b $PAGES_BRANCH origin/$PAGES_BRANCH || echo "Pages branch not deployed yet."
git checkout $7

####### GENERATE NAV SPEC #######
echo "------ Generate input for mkdocs --------"
python3 $1 -it -ihp --title ${9^^} "./$8.md"
ls ${9^^}
echo "------ Move to docs folder --------"
mv ${9^^}/* docs/
sed -i 's/'${9^^}'\///g' _nav.yml
cat _nav.yml
echo "  - 'Home': 'index.md'" >> mkdocs.yml
cat _nav.yml >> mkdocs.yml
cp docs/0.md docs/index.md
repo_url="https://${3}/${4}.git"
echo ${repo_url}
sed -i 's/##PROJECT/'${9^^}'/g' mkdocs.yml
repo_url=$(echo ${repo_url} | sed 's/\//\\\//g')
sed -i 's/##REPO_URL/'"${repo_url}"'/g' mkdocs.yml
sed -i 's/##REPO_NAME/'${9^^}'/g' mkdocs.yml

sed --version

cat mkdocs.yml

## Create download tab if official baseline version
if [[ ${10} == v* ]]; then
    echo "Adding download tab for version ${10}..."
    mkdir -p "docs/download"
    mv "baseline/$8_${10}.docx" docs/download
    spec_name=$(echo ${8/&/and})
    file_name=$(ls docs/download/*.docx)
    filename_corrected=$(echo ${file_name/&/and})
    mv docs/download/"$8_${10}.docx" ${filename_corrected}
    mv "baseline/$8_${10}.pdf" docs/download
    spec_name=$(echo ${8/&/and})
    file_name=$(ls docs/download/*.pdf)
    filename_corrected=$(echo ${file_name/&/and})
    mv docs/download/"$8_${10}.pdf" ${filename_corrected}
    mv "baseline/$8_${10}.epub" docs/download
    spec_name=$(echo ${8/&/and})
    file_name=$(ls docs/download/*.epub)
    filename_corrected=$(echo ${file_name/&/and})
    mv docs/download/"$8_${10}.epub" ${filename_corrected}
    echo ${spec_name}
    #for x in docs/download/*"&"*; do   mv -- "$x" "${x//&/and}"; done
    sed -i 's/PROJECT/'${9^^}'/g' docs/download/index.md
    sed -i 's/SPECWORD/'"${spec_name}_${10}.docx"'/g' docs/download/index.md
    sed -i 's/SPECPDF/'"${spec_name}_${10}.pdf"'/g' docs/download/index.md
    sed -i 's/SPECEPUB/'"${spec_name}_${10}.epub"'/g' docs/download/index.md
    sed -i 's/VERSION/'${10}'/g' docs/download/index.md
    echo "  - Download: 'download/index.md'" >> mkdocs.yml
fi

if [ ${11} == "false" ]; then
    echo "------ Generating site --------"
    echo "Generating docs version ${10}..."
    majorVersion=$(echo ${10} | grep -o v[0-9]*\. | grep -o [0-9]*)
    interVersion=$(echo ${10} | cut -d. -f2)
    minorVersion=$(echo ${10} | cut -d. -f3)
    echo "Major version is ${majorVersion}"
    if [ -n "${majorVersion}" ]; then
        releaseVersion="latestR${majorVersion}"
        echo "Handling release ${releaseVersion}"
    fi

    if [ -z "${majorVersion}" ] || [ -z "${interVersion}" ] || [ -z "${minorVersion}" ]; then
        echo "Deploying non-versioned documents..."
        mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH ${10}
    else
      isPagesSet=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH)
      if [ -z "${isPagesSet}" ]; then
          mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion} latest
          mike set-default --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH latest #That's only needed the first time gl-pages is created
      else
          latestVersionDeployed=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH | grep ${releaseVersion} | cut -d" " -f1)
          latestVersionDeployed_inter=$(echo ${latestVersionDeployed} |cut -d. -f2 )
          latestVersionDeployed_minor=$(echo ${latestVersionDeployed} |cut -d. -f3 )
          latestRelease=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH | grep latest\, | grep -o latestR[0-9]*)
          echo "Latest release is ${latestRelease}"
          echo "Latest deployed version for this release is ${latestVersionDeployed}"

          ##Handling versions: update latestRelease and latest (majorVersion updated)
          if [[ "${releaseVersion}" > "${latestRelease}" ]]; then
              echo "Upgrade latest release...deploying version ${10} as latest and ${releaseVersion}"
              mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion} latest
          elif [[ "${releaseVersion}" == "${latestRelease}" ]]; then
              if [[ ${latestVersionDeployed_inter} == ${interVersion} ]]; then
                if [[ ${latestVersionDeployed_minor} < ${minorVersion} || ${latestVersionDeployed_minor} == ${minorVersion} ]]; then
                  echo "Upgrade latest release...deploying version ${10} as latest and ${releaseVersion}"
                  mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion} latest
                  if [[ ${latestVersionDeployed_minor} < ${minorVersion} ]]; then
                    echo "Hidding version ${latestVersionDeployed}..."
                    mike props --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH --set hidden=true "${latestVersionDeployed}"
                  fi
                else
                  echo "Deploying version ${10}"
                  mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10}
                  echo "Hidding version ${10}..."
                  mike props --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH --set hidden=true "${10}"
                fi
              elif [[ ${latestVersionDeployed_inter} < ${interVersion} ]]; then
                echo "Upgrade latest release...deploying version ${10} as latest and ${releaseVersion}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion} latest
              else
                echo "Deploying version ${10}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10}
              fi
          else
            if [[ ${latestVersionDeployed_inter} == ${interVersion} ]]; then
              if [[ ${latestVersionDeployed_minor} < ${minorVersion} || ${latestVersionDeployed_minor} == ${minorVersion} ]]; then
                echo "Deploying version ${10} as ${releaseVersion}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion}
                if [[ ${latestVersionDeployed_minor} < ${minorVersion} ]]; then
                  echo "Hidding version ${latestVersionDeployed}..."
                  mike props --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH --set hidden=true "${latestVersionDeployed}"
                fi
              else
                echo "Deploying version ${10}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10}
                echo "Hidding version ${10}..."
                mike props --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH --set hidden=true "${10}"
              fi
            elif [[ ${latestVersionDeployed_inter} < ${interVersion} ]]; then
                echo "Deploying version ${10} as ${releaseVersion}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10} ${releaseVersion}
            else
                echo "Deploying version ${10}"
                mike deploy --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH -u ${10}
            fi
          fi

          ##Handling versions: hide version with editorial changes (minorVersion updated)
          #if [[ ${latestVersionDeployed_inter} == ${interVersion} ]]; then
          #  if [[ ${latestVersionDeployed_minor} < ${minorVersion} ]]; then
          #    echo "Hidding version ${latestVersionDeployed}..."
          #    mike props --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH --set hidden=true "${latestVersionDeployed}"
          #  fi
          #fi
      fi
    fi
else
    echo "------ Removing docs version ${11}..."
    majorVersion=$(echo ${11} | grep -o v[0-9]*\. | grep -o [0-9]*)
    if [ -n "${majorVersion}" ]; then
        previousVersion=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH | grep v${majorVersion} | grep -A1 ${11} | grep -v ${11})
        releaseVersion="latestR${majorVersion}"
        echo "Major version is ${majorVersion}"
        echo "Previous version is ${previousVersion}"
    fi

    isLatest=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH ${11} | grep -o latest\, )
    isLatestRelease=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH ${11} | grep -o ${releaseVersion} )
    echo "This is isLatest: $isLatest"
    echo "This is isLatestRelease: $isLatestRelease"
    ## Deal with alias latest
    if [ -n "${isLatest}" ]; then
        if [ -n "${previousVersion}" ]; then
            mike alias --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH -u ${previousVersion} ${releaseVersion} latest
            mike props --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH --set hidden=false ${previousVersion}
        else
            # Look previous release (previous major version)
            majorVersion=$(echo $((--majorVersion)))
            previousReleaseVersion=$(mike list --deploy-prefix public -r $HTTPS_REMOTE -b $PAGES_BRANCH | grep latestR${majorVersion} | grep -o v[0-9]*\.[0-9]*\.[0-9]*)
            echo "Previous major version is ${majorVersion}"
            echo "Previous release version is ${previousReleaseVersion}"
            if [ -n "${previousReleaseVersion}" ]; then
                mike alias --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH -u ${previousReleaseVersion} latest
                mike props --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH --set hidden=false ${previousReleaseVersion}
            else
                echo "Warning: No latest version assigned!!!"
            fi
        fi
    elif [ -n "${isLatestRelease}" ]; then
        if [ -n "${previousVersion}" ]; then
            mike alias --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH -u ${previousVersion} ${releaseVersion}
            mike props --deploy-prefix public -r $HTTPS_REMOTE --allow-empty -p -b $PAGES_BRANCH --set hidden=false ${previousVersion}
        else
            echo "Warning: no ${releaseVersion} available!!"
        fi #Deal with moving down a release
    fi

    mike delete --deploy-prefix public -r $HTTPS_REMOTE -p -b $PAGES_BRANCH ${11}
fi

git checkout $PAGES_BRANCH -- public/

echo "Content of public ..."
ls public/

exit 0

